/***************************************************************************
                          plotdlg.cpp  -  description
                             -------------------
    begin                : Sat Jul 22 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "plotdlg.h"
#include "kauditordoc.h"
#include "kfiledialog.h"
#include "kmsgbox.h"

PlotDlg::PlotDlg(KAuditorApp* parent, const char *name) : QDialog(parent,name,true){
	initDialog();
	
	pDoc = parent->getDocument();
	
	// Read variable names
	const Array<String>& vars = parent->getDocument()->mVariableNames;
	for (int i=0; i<vars.size; i++)
		wAccount->insertItem ((CONSTR) vars[i]);

	//////////////////////////////////////////////////////////////////////////////
	// Build graph selection box
	wPlotSelection->setMultiSelection (true);
	wPlotSelection->setSelected (0, true);
	wPlotSelection->insertItem (i18n("Actual test data"));
	
	wPlot->setEnabled (false);

	// Insert result set names	
	mpResultNames = pDoc->listResultNames ();
	for (int i=0; i<mpResultNames->size; i++)
		wPlotSelection->insertItem ((CONSTR) (*mpResultNames)[i][1]);
	
	///////////////////////////////////////////////////////////////////////////////
	
	connect (wPlot, SIGNAL(clicked()), this, SLOT(plot()));
	connect (wPlotSelection, SIGNAL(selectionChanged()), this, SLOT(plotChanged()));
	connect (wAccount, SIGNAL(activated(int)), this, SLOT(plotChanged()));
	connect (wMonthlyChange, SIGNAL(clicked()), this, SLOT(plotChanged()));
	connect (wBoundaryPerC, SIGNAL(valueChanged(int)), this, SLOT(plotChanged()));
	connect (wExit, SIGNAL(clicked()), this, SLOT(accept()));
	
	mpGnuplot = NULL;
}

PlotDlg::~PlotDlg(){
	delete mpResultNames;
	
	if (mpGnuplot)
		pclose (mpGnuplot);
}

void PlotDlg::plotChanged () {
	// Check that there is at least one selected curve
	bool found=false;
	for (unsigned int i=0; i<wPlotSelection->count(); i++)
		if (wPlotSelection->isSelected(i))
			found=true;
	wPlot->setEnabled (found);

	if (mpGnuplot)
		plot ();
}

void PlotDlg::plot () {
	// Start Gnuplot if not already started
	if (!mpGnuplot) {
		mpGnuplot = popen ("gnuplot", "w");
		//mpGnuplot = fopen("/tmp/plot.gp", "w");
	}

	///////////////////////////////////////////////////////////////////////////////////	
	// Determine output device
	if (wOutputDevice->currentItem() > 0) {
		QString filter;
		if (wOutputDevice->currentItem() == 1)
			filter = "*.eps|Encapsulated PostScript (.eps)";
		else
			filter = "*.gif|GIF image (.gif)";
			
		// Ask filename	
		QString filename = KFileDialog::getSaveFileName ("",filter,this, i18n("Select output file"));
		
		fprintf (mpGnuplot, "set output '%s'\n", (CONSTR) filename);
		if (wOutputDevice->currentItem() == 1)
			fprintf (mpGnuplot, "set terminal postscript EPS\n");
		else			
			fprintf (mpGnuplot, "set terminal gif\n");
	} else
		fprintf (mpGnuplot, "set terminal x11\n");
	
	// Get account index from the selection widget
	int account = wAccount->currentItem() + 1;

	////////////////////////////////////////////////////////////////////////////////////////////
	// Set plot parameters

	// Set plot parameters	
	fprintf (mpGnuplot, "set title '%s'\n",	(CONSTR) wAccount->currentText());
	fprintf (mpGnuplot, "set pointsize 2\n");		// Size of error marks
	fprintf (mpGnuplot, "set timefmt '%%y%%m'\n");	// We use YYMM format
	fprintf (mpGnuplot, "set xdata time\n");
	fprintf (mpGnuplot, "set format x '%%y%%m'\n");
	
	// Write a tic for every month.
	fprintf (mpGnuplot, "set xtics (");
	for (int i=(wMonthlyChange->isChecked()?-1:0); i<12; i++) {
		if (i>(wMonthlyChange->isChecked()?-1:0))
			fprintf (mpGnuplot, ",");
		fprintf (mpGnuplot, "'%d'", int(pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+i,0)));
	}
	fprintf (mpGnuplot, ")\n");
	
	// Monthly change plot needs broken lines, which the parametric-mode provides
	if (wMonthlyChange->isChecked())
		fprintf (mpGnuplot, "set parametric\n");
	else
		fprintf (mpGnuplot, "set noparametric\n");

	// Determine the use of error boundary curves
	double boundaryPerC = float(String(wBoundaryPerC->text()))/100.0;
	bool useBoundary = boundaryPerC>0.005;

	// Construct plot command line
	fprintf (mpGnuplot, "plot ");
	bool found = false;
	// Write a curve definition for each curve, plus two boundary lines, if used
	for (int c = useBoundary?-2:0; c<int(wPlotSelection->count()); c++) {
		// Calculate actual index, because negative values of c are for boundary graphs
		int i = (c>0)? c:0;
		if (wPlotSelection->isSelected(i))
			// Draw each curve with lines, and mark large errors with points,
			// if the error boundaries are enabled
			for (int style=0; style<((i&&useBoundary)?2:1); style++) {
				if (found)
					fprintf (mpGnuplot, ", ");
				found = true;
				fprintf (mpGnuplot, "'-' u 1:2 ");
				if (c>=0 && style==0)
					fprintf (mpGnuplot, "title \"%s\" ", i==0? i18n("Actual balance") : (CONSTR) (*mpResultNames)[i-1][1]);
				else
					fprintf (mpGnuplot, "notitle ");
				fprintf (mpGnuplot, "with %s ", style? "points":"lines");
				fprintf (mpGnuplot, "linetype %d", style? 1 : (c==0? -1 : (c>0)? i+1:0)); // Keep line color/dashes always the same
				if (style)
					fprintf (mpGnuplot, " pointtype 1"); // Always use the same point type to mark large errors
			}
	}
	fprintf (mpGnuplot, "\n");

	/////////////////////////////////////////////////////////////////////////////////////////////
	// Plot each curve
	
	// Write curve data points, plus two boundary lines, if enabled
	for (int c = useBoundary?-2:0; c<int(wPlotSelection->count()); c++) {
		// Calculate actual index, because negative values of c are for boundary graphs
		unsigned int i = (c>0)? c:0;
		if (wPlotSelection->isSelected(i))
			// First draw each curve with lines, and then mark large errors with points,
			// if the error boundaries are enabled
			for (int style=0; style< ((i&&useBoundary)?2:1); style++) {
				// Plot curve by type
				if (i==0) { // Actual test data
					// Calculate boundary multiplier
					double bmult = (c==-2)? 1.0+boundaryPerC : (c==-1)? 1.0-boundaryPerC : 1.0;
	   				for (int p=pDoc->mLoaded.patterns-12; p<pDoc->mLoaded.patterns; p++) {
	   					if (wMonthlyChange->isChecked())
	   						fprintf (mpGnuplot, "%d %f\n", int(pDoc->mLoaded.input(p-1,0)), pDoc->mLoaded.input(p-1,account)*bmult);
	   					fprintf (mpGnuplot, "%d %f\n", int(pDoc->mLoaded.input(p,0)), pDoc->mLoaded.input(p,account)*bmult);
	   				}
				} else {
					// Determine access name of the appropriate result matrix
					String dataname;
					const String& longname = (*mpResultNames)[i-1][1];
					for (int j=0; j<mpResultNames->size; j++)
						if (longname == (*mpResultNames)[j][1]) {
							dataname = (*mpResultNames)[j][0];
							break;
						}

					// Get appropriate result set from the document
	   				if (const Matrix* results = &pDoc->mPredictions[dataname]) {
	   					// Print result value month by month
	    				for (int r=0; r<results->rows; r++) {
	    					// Write startpoint of a monthly change
	    					if (style==0 && wMonthlyChange->isChecked())
	    						fprintf (mpGnuplot, "%d %f\n",
	    								 int(pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+r-1,0)), // Year+month
	    								 pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+r-1,account)); // Actual previous month

	    					// Get predicted and actual values  						
	    					double prediction = results->get (r, account-1);
	    					double actual = pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+r,account);
	    					
	    					// Write datapoint, error mark, or endpoint of a monthly change
	    					if (style==0 || (useBoundary && (prediction<actual*(1-boundaryPerC) || prediction>actual*(1+boundaryPerC))))
		    					fprintf (mpGnuplot, "%d %f\n",
		    							 int(pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+r,0)), // Year+month
		    							 results->get (r, account-1));

		    				// Break in the monthly change curve is made by an additional newline  						
	    					if (style==0 && wMonthlyChange->isChecked())
	    						fprintf (mpGnuplot, "\n");
	    				}
					} else
						fprintf (stderr, "KAuditor INTERNAL ERROR: No result matrix for %s\n", (CONSTR) longname);
				}
				// Write End-Of-Dataset mark
				fprintf (mpGnuplot, "e\n");
			}
	}
		
	// Finalize; restore sane settings for gnuplot
	fprintf (mpGnuplot, "set output '-'\n");
	fflush(mpGnuplot); // Must not forget to do this, because pipes are not flushed on newline
}
